package com.jz.dmp.cmdexectool.scheduler.common.task.sync;

import com.alibaba.fastjson.JSONObject;
import com.jz.dmp.cmdexectool.common.constant.CommConstant;
import com.jz.dmp.cmdexectool.common.constant.DatasouceTypeConstant;
import com.jz.dmp.cmdexectool.common.utils.FreeMarkerUtils;
import com.jz.dmp.cmdexectool.controller.bean.DmpProjectConfigInfoDto;
import com.jz.dmp.cmdexectool.entity.DmpSyncingDatasource;
import com.jz.dmp.cmdexectool.mapper.DmpSyncingDatasourceDao;
import com.jz.dmp.cmdexectool.scheduler.common.process.ResourceInfo;
import com.jz.dmp.cmdexectool.scheduler.common.task.AbstractParameters;
import org.springframework.web.servlet.view.freemarker.FreeMarkerConfigurer;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @ClassName: SyncParameters
 * @Description: sync parameters
 * @Author : Bellamy
 * @Date 2021/2/25
 * @Version 1.0
 */
public class SyncParameters extends AbstractParameters {

    /**
     * shell script
     */
    private String script;

    private String taskAppId;

    /**
     * 数据源相关配置
     */
    private String source;

    /**
     * 目标相关配置
     */
    private String sink;

    /**
     * 环境相关配置
     */
    private String env;

    /**
     * ETL相关配置
     */
    private String transform;

    /**
     * waterdropScript脚本
     */
    private String waterdropScript;

    /**
     * resource list
     */
    private List<ResourceInfo> resourceList;

    public SyncParameters(String script, DmpProjectConfigInfoDto projectConfigInfoDto, DmpSyncingDatasourceDao dmpSyncingDatasourceDao, FreeMarkerConfigurer freeMarkerConfig) throws Exception {
        source = "";
        env = "";
        sink = "";
        transform = "";

        this.script = script;

        JSONObject scriptObj = JSONObject.parseObject(script);
        Map<String, Object> scripts = (Map<String, Object>) scriptObj.get("scripts");
        Map<String, Object> sourceObj = (Map<String, Object>) scripts.get("reader"); //来源数据
        Map<String, Object> targetObj = (Map<String, Object>) scripts.get("writer"); //目标数据
        List<Map<String, Object>> mappingObj = (List<Map<String, Object>>) scripts.get("mappingRelation"); //字段映射关系
        //evn
        Map<String, String> envModel = new HashMap<String, String>();
        envModel.put("sparkappname", "Waterdrop");
        env = FreeMarkerUtils.freemakerJson(CommConstant.WATERDROP_FTL_ENV, envModel, freeMarkerConfig);

        //source information
        Integer sourceId = Integer.valueOf((String) sourceObj.get("sourceDbId"));
        DmpSyncingDatasource dmpSyncingDatasource = dmpSyncingDatasourceDao.queryById(sourceId);
        Integer sourceTypeId = dmpSyncingDatasource.getDatasourceType();
        String[] registerTableName = ((String) sourceObj.get("registerTableName")).split(",");

        //target information
        Integer targetDbId = Integer.valueOf((String) targetObj.get("targetDbId"));
        DmpSyncingDatasource targetDatasource = dmpSyncingDatasourceDao.queryById(targetDbId);
        Integer targetTypeId = dmpSyncingDatasource.getDatasourceType();

        //Jdbc(MySQL、Oracle、SqlServer、PostgreSQL、Informix、DB2)
        if ((sourceTypeId == DatasouceTypeConstant.MySQL || sourceTypeId == DatasouceTypeConstant.Oracle || sourceTypeId == DatasouceTypeConstant.SqlServer || sourceTypeId == DatasouceTypeConstant.PostgreSQL
                || sourceTypeId == DatasouceTypeConstant.Informix || sourceTypeId == DatasouceTypeConstant.DB2)) {
            //source
            for (String tableName : registerTableName) {
                Map jdbcModel = new HashMap();
                jdbcModel.put("driver", dmpSyncingDatasource.getDriverClassName());
                jdbcModel.put("url", dmpSyncingDatasource.getJdbcUrl());
                jdbcModel.put("table", tableName);
                jdbcModel.put("result_table_name", tableName);
                jdbcModel.put("user", dmpSyncingDatasource.getUserName());
                jdbcModel.put("password", dmpSyncingDatasource.getPassword());
                jdbcModel.put("partitionColumn", "");
                jdbcModel.put("numPartitions", "");
                jdbcModel.put("lowerBound", "");
                jdbcModel.put("upperBound", "");
                source = source + FreeMarkerUtils.freemakerJson(CommConstant.WATERDROP_FTL_SOURCE_JDBC, jdbcModel, freeMarkerConfig);
            }
        }
        //Jdbc(MySQL、Oracle、SqlServer、PostgreSQL、Informix、DB2)
        if ((targetTypeId == DatasouceTypeConstant.MySQL || targetTypeId == DatasouceTypeConstant.Oracle || targetTypeId == DatasouceTypeConstant.SqlServer || targetTypeId == DatasouceTypeConstant.PostgreSQL
                || targetTypeId == DatasouceTypeConstant.Informix || targetTypeId == DatasouceTypeConstant.DB2)) {
            //sink
            Map jdbcSinkModel = new HashMap();
            jdbcSinkModel.put("driver", targetDatasource.getDriverClassName());
            jdbcSinkModel.put("url", targetDatasource.getJdbcUrl());
            //# 存储模式，支持overwrite、append、update、ignore、error
            jdbcSinkModel.put("save_mode", targetObj.get("targetInsertMergeOverwrite"));
            //当存储模式是 overwrite时，仅清除表中数据
            if (null != targetObj.get("targetInsertMergeOverwrite")) {
                if ("overwrite".equals(targetObj.get("targetInsertMergeOverwrite"))) {
                    jdbcSinkModel.put("truncate", "true");
                } else {
                    jdbcSinkModel.put("truncate", "false");
                }
            }
            jdbcSinkModel.put("dbtable", targetObj.get("targetTable")); //目标表
            jdbcSinkModel.put("user", targetDatasource.getUserName());
            jdbcSinkModel.put("password", targetDatasource.getPassword());
            sink = FreeMarkerUtils.freemakerJson(CommConstant.WATERDROP_FTL_SINK_JDBC, jdbcSinkModel, freeMarkerConfig);
        }
        //transform
        if (mappingObj.size() > 0 && null != mappingObj) {
            String sourceField = "";
            String targetField = "";
            for (Map<String, Object> item : mappingObj) {
                sourceField += "," + item.get("sourceField");
                targetField += "," + item.get("sourceField") + " as " + item.get("targetField");
            }
            //for (String tableName : registerTableName) {
                Map sqlModel = new HashMap();
                StringBuilder sql = new StringBuilder()
                        .append(" select ")
                        .append(" " + sourceField.substring(1) + " ")
                        .append(" from ")
                        .append(" $t ");
                        //.append(" " + registerTableName[0]);
                sqlModel.put("sql", sql);
                //String table_name = registerTableName[0] + "_view";
                //sqlModel.put("table_name", table_name);
                transform = transform + FreeMarkerUtils.freemakerJson(CommConstant.WATERDROP_FTL_TRANSFORM_SQL, sqlModel, freeMarkerConfig);

                Map targetModel = new HashMap();
                StringBuilder targetSql = new StringBuilder()
                        .append(" select ")
                        .append(" " + targetField.substring(1) + " ")
                        .append(" from ")
                        .append(" $t ");
                        //.append(" " + table_name);
                targetModel.put("sql", targetSql);
                transform = transform + FreeMarkerUtils.freemakerJson(CommConstant.WATERDROP_FTL_TRANSFORM_SQL, targetModel, freeMarkerConfig);
            //}
        }
        /*if (mappingObj.size() > 0 && null != mappingObj) {
            for (Map<String, Object> item : mappingObj) {
                Map<String, String> transformJson2Model = new HashMap<String, String>();
                if (null != item.get("sourceField")) {
                    transformJson2Model.put("source_field", (String) item.get("sourceField"));
                }
                if (null != item.get("targetField")) {
                    transformJson2Model.put("target_field", (String) item.get("targetField"));
                }
                transform = transform + FreeMarkerUtils.freemakerJson(CommConstant.WATERDROP_FTL_TRANSFORM_JSON2, transformJson2Model, freeMarkerConfig);
            }
        }*/

        if (sourceTypeId == DatasouceTypeConstant.Elasticsearch) {
            //source
            for (String tableName : registerTableName) {
                List<String> list = new ArrayList<>();
                String hosts = dmpSyncingDatasource.getHost() + ":" + dmpSyncingDatasource.getPort();
                list.add(hosts);
                Map jdbcModel = new HashMap();
                jdbcModel.put("hosts", list);
                jdbcModel.put("result_table_name", tableName);
                //jdbcModel.put("index", "");
                //jdbcModel.put("name_age", "");
                source = source + FreeMarkerUtils.freemakerJson(CommConstant.WATERDROP_FTL_SOURCE_ELASTICSEARCH, jdbcModel, freeMarkerConfig);
            }
        }
        if (sourceTypeId == DatasouceTypeConstant.Hive) {
            //source
        }
        if (sourceTypeId == DatasouceTypeConstant.Kudu) {
            //source
        }
        if (sourceTypeId == DatasouceTypeConstant.SFTP) {
            //source
            for (String tableName : registerTableName) {
                Map sftpModel = new HashMap();
                sftpModel.put("host", dmpSyncingDatasource.getHost()); //主机名
                sftpModel.put("user", dmpSyncingDatasource.getUserName()); //用户
                sftpModel.put("password", dmpSyncingDatasource.getPassword());
                sftpModel.put("port", dmpSyncingDatasource.getPort()); //端口
                if (null != sourceObj.get("fileType")) {
                    sftpModel.put("fileType", sourceObj.get("fileType").toString()); //文件类型
                }
                sftpModel.put("delimiter", sourceObj.get("sourceCsvDelimiter")); //分隔符
                sftpModel.put("head", "true"); //是否加载第一行
                sftpModel.put("multiLine", "true"); //多行解析
                if (null != sourceObj.get("sourceFtpDir")) {
                    sftpModel.put("path", sourceObj.get("sourceFtpDir").toString()); //文件路径
                }
                sftpModel.put("result_table_name", tableName); //spark生成的临时表名
                source = source + FreeMarkerUtils.freemakerJson(CommConstant.WATERDROP_FTL_SOURCE_SFTP, sftpModel, freeMarkerConfig);
            }
        }
        //waterdrop script
        Map<String, String> waterdropModel = new HashMap<String, String>();
        waterdropModel.put("env", env);
        waterdropModel.put("source", source);
        waterdropModel.put("transform", transform);
        waterdropModel.put("sink", sink);
        waterdropScript = FreeMarkerUtils.freemakerJson(CommConstant.WATERDROP_FTL, waterdropModel, freeMarkerConfig);
    }

    @Override
    public boolean checkParameters() {
        return waterdropScript != null && !waterdropScript.isEmpty();
    }

    /**
     * get project resource files list
     *
     * @return resource files list
     */
    @Override
    public List<ResourceInfo> getResourceFilesList() {
        return resourceList;
    }

    public String getScript() {
        return script;
    }

    public void setScript(String script) {
        this.script = script;
    }

    public String getTaskAppId() {
        return taskAppId;
    }

    public void setTaskAppId(String taskAppId) {
        this.taskAppId = taskAppId;
    }

    public String getSource() {
        return source;
    }

    public void setSource(String source) {
        this.source = source;
    }

    public String getSink() {
        return sink;
    }

    public void setSink(String sink) {
        this.sink = sink;
    }

    public String getEnv() {
        return env;
    }

    public void setEnv(String env) {
        this.env = env;
    }

    public String getTransform() {
        return transform;
    }

    public void setTransform(String transform) {
        this.transform = transform;
    }

    public String getWaterdropScript() {
        return waterdropScript;
    }

    public void setWaterdropScript(String waterdropScript) {
        this.waterdropScript = waterdropScript;
    }

    public List<ResourceInfo> getResourceList() {
        return resourceList;
    }

    public void setResourceList(List<ResourceInfo> resourceList) {
        this.resourceList = resourceList;
    }
}
